@@ -11,7 +11,7 @@
  * called by a name other than "ssh" or "Secure Shell".
  */
 
-/* RCSID("$OpenBSD: servconf.h,v 1.41 2001/04/13 22:46:53 beck Exp $"); */
+/* RCSID("$OpenBSD: servconf.h,v 1.49 2001/08/17 18:59:47 stevesk Exp $"); */
 
 #ifndef SERVCONF_H
 #define SERVCONF_H
@@ -52,7 +52,6 @@ typedef struct {
 						 * for RhostsRsaAuth */
 	int     print_motd;	/* If true, print /etc/motd. */
 	int	print_lastlog;	/* If true, print lastlog */
-	int     check_mail;	/* If true, check for new mail. */
 	int     x11_forwarding;	/* If true, permit inet (spoofing) X11 fwd. */
 	int     x11_display_offset;	/* What DISPLAY number to start
 					 * searching at */
@@ -73,7 +72,7 @@ typedef struct {
 	int     hostbased_uses_name_from_packet_only; /* experimental */
 	int     rsa_authentication;	/* If true, permit RSA authentication. */
 	int     pubkey_authentication;	/* If true, permit ssh2 pubkey authentication. */
-#ifdef KRB4
+#if defined(KRB4) || defined(KRB5)
 	int     kerberos_authentication;	/* If true, permit Kerberos
 						 * authentication. */
 	int     kerberos_or_local_passwd;	/* If true, permit kerberos
@@ -84,15 +83,17 @@ typedef struct {
 	int     kerberos_ticket_cleanup;	/* If true, destroy ticket
 						 * file on logout. */
 #endif
-#ifdef AFS
-	int     kerberos_tgt_passing;	/* If true, permit Kerberos tgt
+#if defined(AFS) || defined(KRB5)
+	int     kerberos_tgt_passing;	/* If true, permit Kerberos TGT
 					 * passing. */
+#endif
+#ifdef AFS
 	int     afs_token_passing;	/* If true, permit AFS token passing. */
 #endif
 	int     password_authentication;	/* If true, permit password
 						 * authentication. */
 	int     kbd_interactive_authentication;	/* If true, permit */
-	int     challenge_reponse_authentication;
+	int     challenge_response_authentication;
 	int     permit_empty_passwd;	/* If false, do not permit empty
 					 * passwords. */
 	int     use_login;	/* If true, login(1) is used */
@@ -120,25 +121,18 @@ typedef struct {
 					 * see if it's still there 
 					 */
 	int	client_alive_count_max;	/*
-					 *If the client is unresponsive
-					 * for this many intervals, above
-					 * diconnect the session 
+					 * If the client is unresponsive
+					 * for this many intervals above,
+					 * disconnect the session 
 					 */
 
+	char   *authorized_keys_file;	/* File containing public keys */
+	char   *authorized_keys_file2;
+
 }       ServerOptions;
-/*
- * Initializes the server options to special values that indicate that they
- * have not yet been set.
- */
-void    initialize_server_options(ServerOptions * options);
 
-/*
- * Reads the server configuration file.  This only sets the values for those
- * options that have the special value indicating they have not been set.
- */
-void    read_server_config(ServerOptions * options, const char *filename);
-
-/* Sets values for those values that have not yet been set. */
-void    fill_default_server_options(ServerOptions * options);
+void	 initialize_server_options(ServerOptions *);
+void	 read_server_config(ServerOptions *, const char *);
+void	 fill_default_server_options(ServerOptions *);
 
 #endif				/* SERVCONF_H */
